==> gecaso-0.2.0.tar.gz: gecaso-0.2.0/gecaso/cache.py <==
import asyncio

from . import utils
from . import storage


def cached(cache_storage, loop=None, **params):
    """ Wraps function to cache its value in cache_storage.

    Positional arguments:
    cache_storage -- subclass of gecaso.BaseStorage used to store cached data

    Keyword arguments:
    loop -- instance of event loop. asyncio.get_event_loop called otherwise
    **params -- passed to cache_storage.set each time its called
    """
    loop = loop or asyncio.get_event_loop()
    if not isinstance(cache_storage, storage.BaseStorage):
        raise ValueError('Provided storage is not subclass of base storage')
    return _cached(cache_storage, loop, **params)


def _cached(cache_storage, loop, **params):
    storage_get = utils.asyncify(cache_storage.get)
    storage_set = utils.asyncify(cache_storage.set)

    def wrapper(function):
        async def wrapped_function(*args, **kwargs):
            key = utils.make_key(function, *args, **kwargs)
            try:
                result = await storage_get(key)
            except KeyError:
                result = await function(*args, **kwargs)
                await storage_set(key, result, **params)
            return result

        def sync_wrapped_function(*args, **kwargs):
            return loop.run_until_complete(wrapped_function(*args, **kwargs))

        if utils.is_async_function(function):
            return utils.wrap(function, wrapped_function)
        else:
            function = utils.asyncify(function)
            return utils.wrap(function, sync_wrapped_function)

    return wrapper

==> gecaso-0.3.1.tar.gz: gecaso-0.3.1/gecaso/cache.py <==
import asyncio

from . import utils
from . import storage


def cached(cache_storage, loop=None, **params):
    """ Wraps function to cache its value in cache_storage.

    Positional arguments:
    cache_storage -- subclass of gecaso.BaseStorage used to store cached data

    Keyword arguments:
    loop -- instance of event loop. asyncio.get_event_loop called otherwise
    **params -- passed to cache_storage.set each time its called
    """
    loop = loop or asyncio.get_event_loop()
    if not isinstance(cache_storage, storage.BaseStorage):
        raise ValueError('Provided storage is not subclass of base storage')
    return _cached(cache_storage, loop, **params)


def _cached(cache_storage, loop, **params):
    def wrapper(function):
        current_calls = dict()

        async def wrapped_function(*args, **kwargs):
            key = utils.make_key(function, *args, **kwargs)
            try:
                result = await cache_storage.get(key)
            except KeyError:
                if not current_calls.get(key):
                    future = asyncio.ensure_future(function(*args, **kwargs),
                                                   loop=loop)
                    current_calls[key] = future
                result = await current_calls[key]
                current_calls.pop(key, None)
                await cache_storage.set(key, result, **params)
            return result

        def sync_wrapped_function(*args, **kwargs):
            return loop.run_until_complete(wrapped_function(*args, **kwargs))

        if utils.is_async_function(function):
            return utils.wrap(function, wrapped_function)
        else:
            function = utils.asyncify(function)
            return utils.wrap(function, sync_wrapped_function)

    return wrapper
